/********************************************/
/* Small On-line Ramsey Number R*(K_3,K_4)  */
/* red  K_3 (colour 1)                      */
/* blue K_4 (colour 2)                      */
/********************************************/
/* Author: Pawel Pralat                     */
/********************************************/
/* Is the number less than or equal to k ?  */
/* n - maximum number of vertices           */
/********************************************/

#include <stdio.h>
#include <time.h>

int ver, k, n, nr_vertices; 
int nr_edges[3];
char v[20][20];                          // number of nodes MUST be smaller than 20
char neighbour[3][20][21];
int nr;
int tmp[20];

inline int is_legal1(int a, int b)
{
  int i,j,k;	

      char * a1 = & neighbour[1][a][0];
      char * b1 = & neighbour[1][b][0];

      if (*a1 > *b1)
	{
	  for (i = 1 ; i <= *b1 ; i++)
	    if (v[a][b1[i]] == 1)
	      return 0;
	}
      else
        {
          for (i = 1 ; i <= *a1 ; i++)
            if (v[b][a1[i]] == 1)
              return 0;
        }
  return 1;
}

inline int is_legal2(int a, int b)
{
  int i,j,k;

      char * a1 = & neighbour[2][a][0];
      char * b1 = & neighbour[2][b][0];

      k = 0;
      if (*a1 > *b1)
        {
          for (i = 1 ; i <= *b1 ; i++)
	    if (v[a][b1[i]] == 2)
	      {
		for (j = 0 ; j < k ; j++)
		  if (v[ tmp[j] ][ b1[i] ] == 2)
		    return 0;
		tmp[k++] = b1[i];
	      } 
	}
      else
        {
          for (i = 1 ; i <= *a1 ; i++)
            if (v[b][a1[i]] == 2)
              {
                for (j = 0 ; j < k ; j++)
                  if (v[ tmp[j] ][ a1[i] ] == 2)
                    return 0;
                tmp[k++] = a1[i];
              }
        }
  return 1;
}

int add()
{
  int condition = nr_edges[1]+nr_edges[2];
  if (2-nr_edges[1] > 0) condition += 2-nr_edges[1];
  if (5-nr_edges[2] > 0) condition += 5-nr_edges[2];
  if (condition >= k) return 0;

  int i, j, c1, c2;

  // new edge between existing nodes
  for (i=0 ; i<nr_vertices-1 ; i++)
    for (j=i+1 ; j<nr_vertices ; j++) 
      if (!v[i][j])          // the builder draws edge {i,j}
	{
	  if (is_legal1(i,j))
	    {
	      v[j][i]=v[i][j]=1;
	      nr_edges[1]++;
	      neighbour[1][i][ ++neighbour[1][i][0] ]=j;
              neighbour[1][j][ ++neighbour[1][j][0] ]=i;

	      c1=add();

	      nr_edges[1]--;
              v[j][i]=v[i][j]=0;
              --neighbour[1][i][0];
              --neighbour[1][j][0];
	    }
	  else
	    c1=1;

	  if (c1)
	    {
	      if (is_legal2(i,j))
		{
		  v[j][i]=v[i][j]=2;
		  nr_edges[2]++;
		  neighbour[2][i][ ++neighbour[2][i][0] ]=j;
		  neighbour[2][j][ ++neighbour[2][j][0] ]=i;

		  c2=add();

		  nr_edges[2]--;
		  v[j][i]=v[i][j]=0;
		  --neighbour[2][i][0];
		  --neighbour[2][j][0];
		}
	      else
		c2=1;
	    }
	  
	  if (c1 && c2)
	    return 1;
	}

  // new edge between existing node and a new one
  if (nr_vertices < n)
    for (i=0 ; i<nr_vertices ; i++) // the builder draws edge {i,nr_vertices}
      {
	j = nr_vertices;

	v[j][i]=v[i][j]=1;
	nr_edges[1]++;
	nr_vertices++;
	neighbour[1][i][ ++neighbour[1][i][0] ]=j;
	neighbour[1][j][ ++neighbour[1][j][0] ]=i;
	
	c1=add();
	
	nr_edges[1]--;
	nr_vertices--;
	v[j][i]=v[i][j]=0;
	--neighbour[1][i][0];
	--neighbour[1][j][0];

	if (c1)
	  {
	    v[j][i]=v[i][j]=2;
	    nr_edges[2]++;
	    nr_vertices++;
	    neighbour[2][i][ ++neighbour[2][i][0] ]=j;
	    neighbour[2][j][ ++neighbour[2][j][0] ]=i;

	    c2=add();

	    nr_edges[2]--;
	    nr_vertices--;
	    v[j][i]=v[i][j]=0;
	    --neighbour[2][i][0];
	    --neighbour[2][j][0];
	  }
	
	if (c1 && c2)
            return 1;
      }

  // new edge between new nodes
  if (nr_vertices < n-1)
    {
      i = nr_vertices;                   // the builder draws edge {nr_vertices,nr_vertices+1}
      j = nr_vertices+1;
      v[j][i]=v[i][j]=1;
      neighbour[1][i][ ++neighbour[1][i][0] ]=j;
      neighbour[1][j][ ++neighbour[1][j][0] ]=i;
	  
      nr_vertices+=2; nr_edges[1]++;
      c1=add();
      nr_edges[1]--; nr_vertices-=2;
      --neighbour[1][i][0];
      --neighbour[1][j][0];

      if (c1)
	{
	  v[j][i]=v[i][j]=2;
	  neighbour[2][i][ ++neighbour[2][i][0] ]=j;
	  neighbour[2][j][ ++neighbour[2][j][0] ]=i;

	  nr_edges[2]++; nr_vertices+=2;
	  c2=add();
	  nr_edges[2]--; nr_vertices-=2;
	  --neighbour[2][i][0];
	  --neighbour[2][j][0];
	}
      
      v[j][i]=v[i][j]=0;
      if (c1 && c2)
	return 1;
    }

  return 0;
}

main(int argc, char * argv[])
{
  if (argc != 6)
    {
      printf("Usage: ramsey <k> <n> <number of the initial graph, 1-?> <input file name> <output file name>\n\n");
      return 0;
    }

  time_t time_begin = time(NULL);

  k = atol(argv[1]);                        // number of rounds
  n = atol(argv[2]);                        // number of vertices
  ver = atol(argv[3]);
  int a,b,c;
  nr_edges[1]=nr_edges[2]=0;

  FILE * in = fopen(argv[4], "rt");
  FILE * out;
  int result;
  int flag = 1;
  int i,j;

  while (flag)
    {
      int tmp;
      fscanf(in, "%d m=%d n=%d ", &tmp, &nr, &nr_vertices);
      
      if (tmp == ver)
	{
	  // number of nodes could be illegal
	  if (nr_vertices > n)
	    {
	      fclose(in);
	      out = fopen(argv[5],"w");
	      
	      fprintf(out, "1000 %d : ILLEGAL (#VERTICES)\n", ver);
	      fclose (out);
	      return 0;
	    }

	  flag = 0;
	  for (i=0 ; i<nr ; i++)
	    {
              fscanf(in, "%d,%d,%d ", &a, &b, &c);

	      // graph could be illegal, that is, could contain red (1) K_3 or blue (2) K_4
            if (c==1)
            {
              if (!is_legal1(a,b))
                {
                  fclose(in);
                  out = fopen(argv[5],"w");
		  
                  fprintf(out, "%d %d : ILLEGAL\n", nr, ver);
                  fclose (out);
                  return 0;
                }
            } 
            else
            {
              if (!is_legal2(a,b))
                {
                  fclose(in);
                  out = fopen(argv[5],"w");

                  fprintf(out, "%d %d : ILLEGAL\n", nr, ver);
                  fclose (out);
                  return 0;
                }
            }

              nr_edges[c]++;
              v[a][b] = c;
              v[b][a] = c;
	      neighbour[c][a][ ++neighbour[c][a][0] ]=b;
	      neighbour[c][b][ ++neighbour[c][b][0] ]=a;
	    }
	}
      else
	{
          for (i=0 ; i<nr ; i++)
	    fscanf(in, "%d,%d,%d ", &a, &b, &c);
	}
      
      fscanf(in, "\n");
    }

  fclose(in);

  result = add();
    if (result)
    {
      out = fopen(argv[5],"w");
      fprintf(out, "%d %d : OK ", k, ver);
    }
  else
    {
      out = fopen(argv[5],"w");
      fprintf(out, "%d %d : NO ", k, ver);
    }

  time_t time_end = (time(NULL) - time_begin);
  float how_long = time_end / 3600.0;

  fprintf(out,"%f \n", how_long);
  fclose(out);

  return 0;
}
